#!/usr/bin/ruby -w
# Auteur <julien@vaubourg.com> pour le CRI de INPL
# 2010 - Centreon 2.0.2

# Formatage d'une configuration Nagios dans le but de la rendre compatible avec les imports Centreon.
# Prend en arguments les fichiers .cfg a prendre en compte, dans n'importe quel ordre.
# Cree les nouveaux fichiers .cfg dans le repertoire de la variable dirTo (en bas).

# Ce que le script fait :
# - Tri les definitions selon leur type, et les range dans des fichiers de meme nom, en differenciant les templates (permet d'importer les differents
# elements dans l'ordre que Centreon preconise)
# - Reformate parfaitement la conf (indentation parfaite, espaces en trop, commentaires inutiles, lignes superflues, etc), pour eviter les
# bugs lorsque Centreon importe
# - Ajoute les attributs name/service_description aux services, obligatoires pour Centreon
# - Recupere les differents attributs *groups pour donner les membres d'un groupe directement au groupe lui-meme (attribut members),
# Centreon n'etant capable de faire les relations que dans ce sens
# - Injecte la configuration des templates de contacts directement dans la definition des contacts (Centreon ne connait pas les tpl de contact)

# Ce que le script ne fait pas :
# - Verifier les doublons d'attributs si il y a plusieurs use pour un contact
# - Gerer les servicegroups correctement

# Procedure d'importation dans Centreon :
# - Administration > Nagios > Load
#
# 1) Cocher Yes pour <Delete all configuration for the chosen type of files>
# - Cocher <Template based method file>
# - Cocher <Notification Command>
# - Copier-coller le contenu notifyCommands.cfg dans le <Manual Filling>
# - Cocher Yes pour <Run Nagios debug (-v)>
# - Cliquer sur <Load> et verifier que le nombre de commandes qu'il dit avoir pris en compte
# correspond bien au nombre de commandes a importer (grep -c 'define command' notifyCommands.cfg)
#
# 2) Recommender, en cochant No a <Delete all configuration for the chosen type of files>
# - Et en cochant <Check Command>, le reste ne change pas
# - Copier-coller le contenu des fichiers qui restent (un par un, ou a la suite dans le champ) dans cet ordre :
# <commands timeperiods contacts contactgroups tplHosts hosts hostgroups tplServices services servicegroups serviceescalations>
# - Toujours verifier le nombre d'insertion : Centreon ne dit rien quand il ignore une definition

# Un script bash accompagne ce script pour aider a importer les fichiers.


#########################
######## FUNCTIONS ######
#########################

# Creation du dossier qui recuperera les fichiers convertis
def createDirTo(dirTo)
	Dir.mkdir dirTo unless File.exist? dirTo

	unless File.directory? dirTo
		puts "ERREUR: #{dirTo} n'est pas un dossier ou n'a pas pu etre cree."
		exit 1
	end
end

# Purge du dossier qui recois les fichiers convertis
def emptyDir(dirTo)

	# Si le dossier n'est pas vide
	if Dir.entries(dirTo).length > 2
		printf "Purger #{dirTo}/ (Y/n) ? "

		# Pour chaque fichier, le supprimer, si l'utilisateur donne sont accord (oOyY)
		Dir.foreach(dirTo) do |f|
			f = "#{dirTo}/#{f}"
			File.delete(f) if File.file? f
		end if $stdin.gets.chomp =~ /^[yo]?$/i
	end
end

# Reformatage des fichiers actuels
# (indentation correcte, tabulations partout, suppression des lignes inutiles)
# Centreon bug a l'import si le fichier n'est pas parfait.
def reformat(conf)

	# Suppression de tous les commentaires
	conf.gsub!(/[[:space:]]*[#;].*$/, "")

	# Suppression de tous les espaces/tab de debutde ligne
	conf.gsub!(/^[[:space:]]+/, "")

	# Suppression de tous les espaces/tab de fin de ligne
	conf.gsub!(/[[:space:]]+$/, "")

	# Remplacement de toutes les tabulations par des espaces
	conf.gsub!(/\t/, " ")

	# Remplacement des blocs d'espaces par une seule espace
	conf.gsub!(/[ ]+/, " ")

	# Suppression des lignes vides
	conf.gsub!(/^ ?\n/m, "")

	# Indentation des definitions
	conf.gsub!(/^((?!define|\}).*)$/, "\t\\1")

	# Ajout d'une ligne vierge entre chaque def.
	conf.gsub!(/^\}$/, "}\n")

	# Ajout d'une espace avant les accolades
	conf.gsub!(/([^ ])\{/, "\\1 {")

	return conf
end

# Extraction des membres des groupes.
# Si une entite contient un attribut de groupe, on le stocke dans un tableau pour ce groupe et on
# lui retire son attribut. Si un groupe contient un attribut members, on extrait aussi ses membres.
# A terme, le but est de recreer proprement les definitions des groupes, qui declarent eux-meme
# leurs membres (Centreon ne supporte les relations que dans ce sens, sinon il ne les prend pas en compte a l'import).
def saveMembers(definition, type)
	group = {}

	# Extraction du nom de l'entite (attribut name pour les services, type_name sinon)
	name = definition.match(/^\t#{type != "service" && type != "serviceescalation" ? "#{type}_" : ""}name (.*)/)[1]

	# Nous avons affaire a un groupe : il faudra alors isoler les members qu'il definit, eventuellement
	if(type =~ /group/)
		if(definition =~ /^\tmembers (.*)/)

			# Pour chaque membre, on l'ajoute dans un hash du nom du groupe
			$1.split(",").each do |member|
				member.strip!
				group[name] = [] unless group[name]
				group[name] << member
			end

			# On vire ensuite la definition du members du groupe, qu'on rajoutera plus tard proprement
			definition.gsub!(/^\tmembers ([^\n]+)\n/m, "")
		end

	# Ce n'est pas un groupe : il faut donc regarder si il se reclame d'un groupe
	else
		if(definition =~ /^\t#{type}groups (.*)/)

			# Pour chaque groupe qu'il declare, on ajoute l'entite au hash portant le nom du groupe correspondant
			$1.split(",").each do |groupName|
				groupName.strip!
				group[groupName] = [] unless group[groupName]
				group[groupName] << name
			end

			# On vire la definition des groupes de l'entite, qui sera reportee directement dans le groupe
			definition.gsub!(/^\t#{type}groups ([^\n]+)\n/m, "")
		end
	end

	# Integration des nouveaux groupes avec leurs membres dans le hash global,
	# trie en fonction du type des groupes (ex: groups['contactgroup']['criSrt'])
	group.each do |g,mbrs|
		groupType = type + (type !~ /group/ ? "group" : "")
		$groups[groupType][g] = [] unless $groups[groupType][g]
		$groups[groupType][g] |= mbrs
	end

	# Retour de la definition de l'entite, sans les *groups et members
	return definition
end

# Parse un lot de definition pour trier les differentes entites par fichiers
# (ex: les contactgroup dans contactgroups.cfg)
# L'interet est d'ensuite pouvoir importer la conf dans Centreon, en rentrant bien les elements
# dans le bon ordre pour qu'il prenne tout en compte.
def injectFiles(conf, dirTo)

	# Pour chaque definition (extraction facile puisque la conf a ete correctement reformatee)
	conf.scan(/^(define ([^ ]+) \{\n[^\}]+\})/m) do |definition, type|

		# Une definition est un template si il y a un register0.
		# Les templates sont ranges dans des fichiers a part.
		template = definition.match("register 0")

		# Centreon fait une distinction que Nagios ne connais pas : le commandes qui servent a notifier
		# et celles qui servent pour les checks. Arbitrairement, on considere que si il y a notify dans le nom
		# de la commande, si c'est une commande de notification, sinon de check
		notify = (type == "command" && definition.match("notify"))

		# Si c'est un template de definition de contact, c'est un cas que Centreon ne supporte pas du tout.
		# On isole donc sa definition dans un hash global pour l'injecter ensuite dans les definition qui dependent
		# de ce template.
		if type == "contact" && template
			name = definition.match("\tname (.*)")[1]
			$tplContacts[name] = definition.gsub(/\t(register 0|name [^\n]+)\n/m, "").match(/\{\n(.*)\n\}/m)[1]

		# Tout les autres definitions
		else
			# Dans le cas d'un service, il faut s'assurer qu'il ne manque aucun element, essentiel a Centreon
			# mais facultatif pour Nagios
			if(type == "service" || type == "serviceescalation")
				name = definition.match(/\tname (.*)/)
				desc = definition.match(/\tservice_description (.*)/)
				tpl = definition.match(/\tuse (.*)/)

				name = name[1] if name
				desc = desc[1] if desc

				# On a extrait le use (template) pour creer un nom de service intelligent, si celui-ci n'en a pas.
				# Quand les services n'ont pas de nom dans Nagios, c'est souvent que le tpl qu'ils utilisent en porte un
				# suffisament explicite.
				tpl = tpl =~ / / ? tpl[1].split(' ').select { |w| w.capitalize! || w } * ' ' : tpl[1] if tpl

				# Si on ne peut pas faire autrement, un nom et une description seront generes avec un chiffre aleatoire
				randId = rand 10000
				insertIndex = definition.index("}") - 1

				# Modification ou ajout du name, selon les cas.
				# Si il existe deja, on remplace ses espaces par des tirets, pour que Centreon accepte de les importer correctment
				definition.sub!(/\tname .*/, "\tname #{ name.gsub(' ', '-') }") if name
				definition.insert(insertIndex, "\n\tname #{ (desc ? desc : (tpl ? "use #{tpl}" : "NAME #{randId}")).gsub(' ', '-') }") unless name
				
				insertIndex = definition.index("}") - 1

				# Idem pour les description : si le service a deja un nom, on le recopie en description, sinon on utilise le nom du tpl,
				# ou bien encore on genere un nom aleatoire
				definition.sub!(/\tservice_description .*/, "\tservice_description #{ desc.gsub(' ', '-') }") if desc
				definition.insert(insertIndex, "\n\tservice_description #{ (name ? name : (tpl ? "use #{tpl}" : "DESC #{randId}")).gsub(' ', '-') }") unless desc
			end

			# Extraction des membres du groupes ou des appartenances aux groupes
			definition = saveMembers(definition, type) if type =~ /^(host|contact|service)(group|escalation)?$/ && !template

			# Creation du nom du fichier qui accueillera cette definition
			newFileName = "#{dirTo}/#{template ? "tpl" + type.capitalize : type}s.cfg" unless notify
			newFileName = "#{dirTo}/notifyCommands.cfg" if notify

			# Ajout de la definition au fichier qui va bien
			File.open(newFileName, "a") { |f| f << "\n" << definition << "\n" }
		end
	end
end

# Injection des membres des groupes, a posteriori
def injectMembers(dirTo)

	# Pour chaque groupe pour lequel des membres ont ete collectes
	$groups.each do |groupType, groups|

		# Selon le type de groupe, on trouve le fichier qui contient les definitions
		fileName = "#{dirTo}/#{groupType}s.cfg"

		if File.exist? fileName
			conf = IO.read fileName
			definitions = ""

			# Puisque le fichier est parfaitement formater, on recupere chaque defininition de groupe
			# en coupant aux lignes blanches
			conf.split(/^$/).each do |definition|

				# Recuperation du nom du groupe
				group = definition.match(/^\t#{groupType}_name (.*)/)[1]

				# Si ce groupe a des membres qui ont ete trouves, on lui injecte son attribut members
				# qui contiendra la liste de ses membres
				if groups[group]
					insertIndex = definition.index("}") - 1
					definition.insert(insertIndex, "\n\tmembers #{groups[group] * ','}")
				end

				# Constitution de la nouvelle liste de groupes, completes
				definitions += definition
			end

			# Mise a jour du fichier
			File.open(fileName, "w") { |f| f << definitions }
		end
	end
end

# Centreon ne supporte pas les templates de contact : durant le parsage,
# on a donc collecte tous les tps de contact, qui ne se retrouve plus dans la conf.
# L'idee est donc maintenant d'injecter la conf. des templates directement dans les contacts
# qui en dependent.
def extendTplContacts(dirTo)
	fileName = "#{dirTo}/contacts.cfg"
	definitions = ""

	if File.exist? fileName
		conf = IO.read fileName

		# Pour chaque contact trouve dans le fichier des contacts
		conf.split(/^$/).each do |definition|

			# Si le contact depend d'un template, on remplace l'appel du tpl par la definition
			# qu'il contient
			if definition =~ /^\tuse (.*) ?/
				definition.gsub!($&, $tplContacts[$1]) if $tplContacts[$1]
			end

			# Constitution de la nouvelle liste de contacts, completes
			definitions += definition
		end
	end

	# Mise a jour du fichier
	File.open(fileName, "w") { |f| f << definitions }
end


#########################
######## PROGRAMME ######
#########################

# Repertoire qui contiendra les fichiers cfg convertis
dirTo = "./toCentreon"

# Creation et purge du dossier, avant remplissage
createDirTo(dirTo)
emptyDir(dirTo)

# Creation des hashs globaux
$groups = { "hostgroup" => {}, "contactgroup" => {}, "servicegroup" => {} }
$tplContacts = {}

# Pour chaque fichier cfg passe en argument
ARGV.each do |fileName|
	unless File.readable? fileName
		puts "ERREUR: Le fichier #{fileName} ne peut etre lu."
	else
		# Recuperation de la conf du fichier, en la reformatant a la volee
		conf = reformat(IO.read(fileName))

		# Decoupage et corrections, classement en fichiers de type
		injectFiles(conf, dirTo)
	end
end

# Injection a posteriori des membres des groupes
injectMembers(dirTo)

# Remplacement des appels de template des contacts par la definition du tpl
extendTplContacts(dirTo)

# Hop!
puts "Fichiers correctement generes dans #{dirTo}/."
