#!/usr/bin/perl -w
# Auteur <julien@vaubourg.com> pour le CRI de INPL
# 2010 - Centreon 2.0.2

# Synchronisation de la base de donnee MySQL Centreon des utilisateurs en fonction du LDAP.
# Ajoute, met a jour ou supprime les utilisateurs et leurs relations en fonction du LDAP.
#
# Gere egalement des groupes d'ACL (vues) de Centreon en fonction des groupes LDAP de l'utilisateur
# (ajoute les groupes et lie les utilisateurs avec, ne les supprime pas et ne delie pas les utilisateurs,
# sauf en cas de suppression de celui-ci, puisque c'est un cas rarissime)
#
# Des groupes de type contactgroup sont crees en parallele et geres de la meme facon
# (ils sont dependant des groupes ACL : si un utilisateur n'appartient plus au contactgroup dans Centreon mais
# appartient toujours au groupe ACL, il ne sera pas rajoute, permettant ainsi une plus grande fluidite)
#
# Les groupes sont reconnus en fonction de leur nom, et les utilisateurs en fonction de leur dn
#
# Relance les Nagios s'il en est besoin.

### Droits detailles de l'utilisateur MySQL synchro (mdp: synchro, 1ere ligne) qui sert pour la connexion en local :
#
# GRANT ALL ON centreon2.contact TO synchro@localhost IDENTIFIED BY 'synchro';
# GRANT LOCK TABLES ON centreon2.* TO synchro@localhost;
# GRANT SELECT ON centreon2.general_opt TO synchro@localhost;
# GRANT SELECT ON centreon2.command TO synchro@localhost;
# GRANT SELECT ON centreon2.timeperiod TO synchro@localhost;
# GRANT SELECT ON centreon2.nagios_server TO synchro@localhost;
# GRANT ALL ON centreon2.contact_hostcommands_relation TO synchro@localhost;
# GRANT ALL ON centreon2.contact_servicecommands_relation TO synchro@localhost;
# GRANT ALL ON centreon2.acl_groups TO synchro@localhost;
# GRANT ALL ON centreon2.acl_group_contacts_relations TO synchro@localhost;
# GRANT ALL ON centreon2.contactgroup TO synchro@localhost;
# GRANT ALL ON centreon2.contactgroup_contact_relation TO synchro@localhost;
# GRANT SELECT,DELETE ON centreon2.contact_host_relation TO synchro@localhost;
# GRANT SELECT,DELETE ON centreon2.contact_param TO synchro@localhost;
# GRANT SELECT,DELETE ON centreon2.contact_service_relation TO synchro@localhost;
#
###

use strict;
use warnings;
use List::Util 'first';
use Net::LDAP;
use DBI;


##
## Divers
##

# Nom de la commande de notification qui sera attribuee par defaut
my $notifyCommand_email = 'host-notify-by-email';

# Mot de passe MySQL pour l'utilisateur synchro
my $pwdMySQLSynchro = '*******';

# Nom de l'attribut qui defini les groupes pour un utilisateurs
my $attrGroup = 'INPLatrGroupe';

# Groupes (attrGroup) du LDAP pour lesquels les utilisateurs seront utilises
my @topGroups = qw( CRI );

# Groupe pour lequel tous les utilisateurs ont automatiquement les pleins pouvoirs
my $admGroup = 'CRI-SRT';

# Definition de la date pour les commentaires de utilisateurs ajoutes/modifies
my ($s, $min, $h, $d, $m, $y) = localtime time;
my $date = sprintf('%02d/%02d/%d a %02dh%02d', $d, $m+1, 1900+$y, $h, $min);

# Mises a jour qui necessitent de relancer les nagios :
# - qui ne concernent pas juste les ACL
# - qui ne sont pas des ajouts, puisque ca signifie qu'elles n'ont pas encore d'utilite
my $updates = 0;

# C'est parti
print "\nSYNCHRONISATION LDAP DU $date\n";


##
## Recuperation des utilisateurs de Centreon
##

# Connexion MySQL
my $mysql = DBI->connect('DBI:mysql:database=centreon2;host=localhost', 'synchro', $pwdMySQLSynchro, { 'RaiseError' => 1 });

# Dump des utilisateurs inscrits dans Centreon qui s'authentifient avec le LDAP
# qui servent pour les vues de Centreon (ACL) comme pour les notifications des Nagios
my $sql = $mysql->prepare("SELECT
	contact_id,
	contact_ldap_dn,
	contact_name,
	contact_alias,
	contact_email
FROM contact WHERE contact_ldap_dn IS NOT NULL");

$sql->execute;
my %contactsSQL;

# Creation d'un hash de hash de tous les utilisateurs Centreon qui ont une authentification LDAP,
# en donnant la correspondance de noms des champs entre le MySQL et le LDAP
while(my $c = $sql->fetchrow_hashref) {
	$contactsSQL{$c->{'contact_ldap_dn'}} = {
		id => $c->{'contact_id'},
		uid => $c->{'contact_alias'},
		displayName => $c->{'contact_name'},
		mail => $c->{'contact_email'}
	};
}


##
## Parcours des utlisateurs du LDAP et mise en conformite du MySQL
##

# Extraction des parameters LDAP depuis l'interface de Centreon (Administration > Options > LDAP)
$sql = $mysql->prepare("SELECT
	ldap_host,
	ldap_port,
	ldap_base_dn
FROM general_opt");

$sql->execute;
my $ldap_params = $sql->fetchrow_hashref;

# Connexion au LDAP
my $ldap = Net::LDAP->new(
	$ldap_params->{'ldap_host'},
	port => $ldap_params->{'ldap_port'},
	version => 3
) or die "ERREUR: Impossible de contacter l'annuaire LDAP ($@)";

# Liaison
my $msg = $ldap->bind;
$msg->code && die "ERREUR: ".$msg->error;

# Parcours des utilisateurs, groupe par groupe (permet de selectionner les branches d'utiliateurs
# interessants et eventuellement diviser les requetes pour en recevoir moins a la fois
foreach my $topGroup (@topGroups) {

	# Recherche
	$msg = $ldap->search(
		base   => $ldap_params->{'ldap_base_dn'},
		scope  => 'sub',
		filter => "(&(objectClass=person)($attrGroup=$topGroup))",
		attrs  => [ 'uid', 'displayName', 'mail', $attrGroup ]
	);

	$msg->code && die "ERREUR: ".$msg->error;
	my @dnInLDAP;

	# Pour chaque utilisateur trouve dans le LDAP, on l'ajoute au MySQL si il n'existe pas, ou bien on modifie ses
	# infos si elles ont changees depuis le LDAP. On le supprime si il n'existe plus dans le LDAP, et on ajoute
	# ses eventuels nouveaux groupes si ils n'existent pas deja. Sinon, on l'ajoute juste dans le groupe.
	foreach my $person ($msg->all_entries) {

		# Une absence d'email est un crime dans Centreon
		my $mail = $person->get_value('mail') ? $person->get_value('mail') : 'no@mail';

		# Recuperation de la liste des groupes auquels appartient l'utilisateur
		my $personGroups = $person->get_value($attrGroup, asref => 1);

		# L'utilisateur LDAP courant n'existe pas : on l'ajoute au MySQL
		unless($contactsSQL{$person->dn}) {
			print "INFO: Ajout de ".$person->dn."\n";

			# Fermeture des tables avant d'ecrire, ca ne coute rien
			$mysql->do("LOCK TABLES
				contact WRITE,
				contact_hostcommands_relation WRITE,
				contact_servicecommands_relation WRITE,
				command READ,
				timeperiod READ
			");

			$mysql->do("SET AUTOCOMMIT = 0");

			# Injection avec les parametres par defaut
			# (notifications 24/7, interface en FR, acces au Centreon sans etre admin, notification en cas de down warning ou critical)
			$mysql->do("INSERT INTO contact (
				timeperiod_tp_id,
				timeperiod_tp_id2,
				contact_alias,
				contact_name,
				contact_passwd,
				contact_lang,
				contact_host_notification_options,
				contact_service_notification_options,
				contact_email,
				contact_pager,
				contact_comment,
				contact_oreon,
				contact_admin,
				contact_type_msg,
				contact_activate,
				contact_auth_type,
				contact_ldap_dn,
				contact_acl_group_list,
				contact_autologin_key,
				contact_charset
			) VALUES (
				(SELECT tp_id FROM timeperiod WHERE tp_name='24x7'), timeperiod_tp_id,
				".$mysql->quote($person->get_value('uid')).", ".$mysql->quote($person->get_value('displayName')).",
				NULL, 'fr_FR','d','w,c',
				".$mysql->quote($mail).", NULL,
				'[Ajout] Synchronisation LDAP du $date',
				'1','".((first { $_ eq $admGroup } @$personGroups) ? 1 : 0)."',NULL,'1','ldap',
				'".$person->dn."',
				NULL,NULL,NULL
			);");

			# Liaison de l'utilisateur avec une commande de notification pour les alertes de services
			$mysql->do("INSERT INTO contact_hostcommands_relation (
				contact_contact_id,
				command_command_id
			) VALUES (
				(SELECT contact_id FROM contact WHERE contact_ldap_dn='".$person->dn."' LIMIT 1),
				(SELECT command_id FROM command WHERE command_name=".$mysql->quote($notifyCommand_email)." AND command_type=1 LIMIT 1)
			)");

			# Liaison de l'utilisateur avec une commande de notification pour les alertes d'hotes
			$mysql->do("INSERT INTO contact_servicecommands_relation (
				contact_contact_id,
				command_command_id
			) VALUES (
				(SELECT contact_id FROM contact WHERE contact_ldap_dn='".$person->dn."' LIMIT 1),
				(SELECT command_id FROM command WHERE command_name=".$mysql->quote($notifyCommand_email)." AND command_type=1 LIMIT 1)
			)");

			# On remballe
			$mysql->do("COMMIT");
			$mysql->do("SET AUTOCOMMIT = 1");
			$mysql->do("UNLOCK TABLES");

		# L'utilisateur a des informations differentes du LDAP au MySQL : on met a jour ce dernier
		} elsif($contactsSQL{$person->dn}->{'uid'} ne $person->get_value('uid') ||
			$contactsSQL{$person->dn}->{'displayName'} ne $person->get_value('displayName') ||
			$contactsSQL{$person->dn}->{'mail'} ne $mail) {

			print "INFO: Modification de ".$person->dn."\n";

			# Mise a jour, avec une trace de debug dans le champ de commentaire de l'utilisateur
			$mysql->do("UPDATE contact SET
				contact_name=".$mysql->quote($person->get_value('uid')).",
				contact_alias=".$mysql->quote($person->get_value('displayName')).",
				contact_email=".$mysql->quote($mail).",
				contact_comment=CONCAT(contact_comment, '\n[Maj] Synchronisation LDAP du $date')
			WHERE contact_ldap_dn='".$person->dn."'");

			$updates++;
		}

		# Parcours de ses groupes : si un de ses groupes n'existe pas, il sera cree, en ACL (vues) et en contactgroup (notifications)
		# Dans tous les cas, l'utilisateur est ajoute dans le groupe ACL si il n'y est pas deja.
		# Le contactgroup est dependant du groupe ACL (on ne rajoute pas l'utilisateur dans le contactgroup si il n'y est plus)
		foreach (@$personGroups) {
			my $isInGroup = 1;
			my $isNewGroup = 0;

			# Le groupe ACL existe-t-il deja ?
			$sql = $mysql->prepare("SELECT COUNT(*) exist FROM acl_groups WHERE acl_group_name=".$mysql->quote($_));
			$sql->execute;

			# Ajout du groupe si ca n'est pas le cas
			unless($sql->fetchrow_hashref->{'exist'}) {

				# Un administrateur ne peut pas etre affecte a un groupe d'ACL (il a deja tous les droits)
				if(! first { $_ eq $admGroup } @$personGroups) {
					print "INFO: Ajout du groupe $_ (ACL)\n";

					# Injection
					$mysql->do("INSERT INTO acl_groups (
						acl_group_name,
						acl_group_alias,
						acl_group_activate
					) VALUES (
						".$mysql->quote($_).",
						".$mysql->quote("$_ (LDAP)").",
						1
					)");
				}

				$sql = $mysql->prepare("SELECT COUNT(*) exist FROM contactgroup WHERE cg_name=".$mysql->quote($_));
				$sql->execute;

				# Le groupe jumeau existe-t-il en contactgroup ? (theoriquement non)
				unless($sql->fetchrow_hashref->{'exist'}) {
					print "INFO: Ajout du groupe $_ (ContactGroup)\n";

					$mysql->do("INSERT INTO contactgroup (
						cg_name,
						cg_alias,
						cg_comment,
						cg_activate
					) VALUES (
						".$mysql->quote($_).",
						".$mysql->quote("$_ (LDAP)").",
						'[Ajout] Synchronisation LDAP du $date',
						'1'
					)");
				}

				# Puisque le groupe vient d'etre cree, inutile de verifier si l'utilisateur est deja dedans
				# Et si c'est un nouveau groupe, inutile de relancer les Nagios pour l'ajout d'utilisateur
				$isNewGroup = 1;

			# Si le groupe existe deja, verification de l'appartenance de notre utilisateur a ce groupe
			} else {

				$sql = $mysql->prepare("SELECT COUNT(*) exist FROM acl_group_contacts_relations
					WHERE contact_contact_id=(SELECT contact_id FROM contact WHERE contact_ldap_dn='".$person->dn."' LIMIT 1)
					AND acl_group_id=(SELECT acl_group_id FROM acl_groups WHERE acl_group_name=".$mysql->quote($_)." LIMIT 1)");

				$sql->execute;
				$isInGroup = $sql->fetchrow_hashref->{'exist'};
			}

			# Ajout de l'utilisateur dans le groupe si il n'y est pas deja
			if(!$isInGroup || $isNewGroup) {

				# Un administrateur ne peut pas etre affecte a un groupe d'ACL (il a deja tous les droits)
				if(! first { $_ eq $admGroup } @$personGroups) {
					print "INFO: Ajout de ".$person->dn." dans $_ (ACL)\n";

					# Injection de la relation utilisateur-groupe ACL
					$mysql->do("INSERT INTO acl_group_contacts_relations (
						contact_contact_id,
						acl_group_id
					) VALUES (
						(SELECT contact_id FROM contact WHERE contact_ldap_dn='".$person->dn."' LIMIT 1),
						(SELECT acl_group_id FROM acl_groups WHERE acl_group_name=".$mysql->quote($_)." LIMIT 1)
					)");
				}

				# L'utilisateur est-il deja rattache au contactgroup ? (theoriquement non)
				$sql = $mysql->prepare("SELECT COUNT(*) exist FROM contactgroup_contact_relation
					WHERE contact_contact_id=(SELECT contact_id FROM contact WHERE contact_ldap_dn='".$person->dn."' LIMIT 1)
					AND contactgroup_cg_id=(SELECT cg_id FROM contactgroup WHERE cg_name=".$mysql->quote($_)." LIMIT 1)");

				$sql->execute;

				# Injection de la relation utilisateur-contactgroup
				unless($sql->fetchrow_hashref->{'exist'}) {
					print "INFO: Ajout de ".$person->dn." dans $_ (ContactGroup)\n";

					$mysql->do("INSERT INTO contactgroup_contact_relation (
						contact_contact_id,
						contactgroup_cg_id
					) VALUES (
						(SELECT contact_id FROM contact WHERE contact_ldap_dn='".$person->dn."' LIMIT 1),
						(SELECT cg_id FROM contactgroup WHERE cg_name=".$mysql->quote($_)." LIMIT 1)
					)");
				}

				# Inutile de prendre l'ajout en consideration pour le reload des nagios si c'est un nouveau groupe
				# (il n'est encore rattache a rien, donc invisible pour les nagios)
				# NOTE: Si le groupe a ete cree pour l'utilisateur d'avant, on considere donc quand meme qu'il est potentiellement utilise
				$updates++ unless $isNewGroup;
			}
		}

		# Tenue d'un tableau de dn (identifiants unique des utilisateurs) du LDAP, pour savoir ensuite
		# qui est dans le MySQL mais n'est plus dans le LDAP
		push(@dnInLDAP, $person->dn);
	}

	# Suppression des utilisateurs qui sont dans MySQL en ayant un dn LDAP attribue, alors qu'il n'existent plus dans le LDAP
	foreach my $dn (keys %contactsSQL) {

		# Si l'utilisateur n'existe pas dans les dn du LDAP
		unless(first {$_ eq $dn} @dnInLDAP) {
			print "INFO: Suppression de $dn\n";

			# Suppression de la table de contacts, de celles servant pour les commandes de notifications rattachees et suppression
			# des appartenances aux groupes
			$mysql->do("DELETE FROM contact WHERE contact_id=".$contactsSQL{$dn}->{'id'});
			$mysql->do("DELETE FROM contact_hostcommands_relation WHERE contact_contact_id=".$contactsSQL{$dn}->{'id'});
			$mysql->do("DELETE FROM contact_servicecommands_relation WHERE contact_contact_id=".$contactsSQL{$dn}->{'id'});
			$mysql->do("DELETE FROM contact_host_relation WHERE contact_id=".$contactsSQL{$dn}->{'id'});
			$mysql->do("DELETE FROM contact_service_relation WHERE contact_id=".$contactsSQL{$dn}->{'id'});
			$mysql->do("DELETE FROM acl_group_contacts_relations WHERE contact_contact_id=".$contactsSQL{$dn}->{'id'});
			$mysql->do("DELETE FROM contactgroup_contact_relation WHERE contact_contact_id=".$contactsSQL{$dn}->{'id'});
			$mysql->do("DELETE FROM contact_param WHERE cp_contact_id=".$contactsSQL{$dn}->{'id'});

			$updates++;
		}
	}

}

# Si il y a eu des modifications du MySQL, on relance tous les Nagios (reload)
# (pour optimiser il faudra ne relancer que lorsque ca concerne des utilisateurs qui ne servent pas juste pour les ACL)
if($updates) {

	# Recuperation des parametres des differentes Nagios
	$sql = $mysql->prepare("SELECT id, name, localhost, init_script FROM nagios_server");
	$sql->execute;

	# Pour chaque Nagios, recharger sa configuration
	while(my $n = $sql->fetchrow_hashref) {
		print "INFO: Rechargement de ".$n->{'name'}."\n";

		# Nagios du Centreon
		if($n->{'localhost'}) {
			my $init = $n->{'init_script'};
			`sudo $init reload`;

		# Nagios satellite
		} else {
			my $nsId = $n->{'id'};
			`echo 'RELOAD:$nsId' >> /var/lib/centreon/centcore.cmd`
		}
	}

# Rien a faire
} else {
	print "INFO: Aucun rechargement de Nagios.\n";
}

# Vous pouvez eteindre votre poste de television et reprendre une activite normale
$ldap->unbind;
$sql->finish;
$mysql->disconnect;
