#!/usr/bin/perl -w
# Auteur <julien@vaubourg.com> pour le CRI de INPL
# 2010 - Centreon 2.0.2

# Rassemble les hosts qui dependent d'un meme service au sein d'un hostgroup lui-meme dependant
# de ce service. Fusionne ensuite les hostgroups identiques, cree les groupes et les relations
# et defait les anciennes.

use strict;
use warnings;
use DBI;

my $mysql = DBI->connect('DBI:mysql:database=centreon2;host=localhost', 'tmp', 'tmp', { 'RaiseError' => 1 });

my (%hostsByHostgroup, %servicesByHostgroup);

# Selection de tous les couples host/services pour lesquels services
# se retrouve partage entre plusieurs hosts
my $sql = $mysql->prepare("SELECT service_service_id service, host_host_id host
			FROM host_service_relation
			WHERE service_service_id IN (SELECT service_service_id
							FROM host_service_relation
							GROUP BY service_service_id
							HAVING COUNT(*) > 1)
			AND host_host_id IS NOT NULL");
$sql->execute;

# Pour chacun de ces couples
while(my $rel = $sql->fetchrow_hashref) {

	# Les hostgroups crees porteront le nom des services et contiendront les hosts concernes par ce service
	$hostsByHostgroup{$rel->{'service'}} = [] unless @hostsByHostgroup{$rel->{'service'}};
	push @{$hostsByHostgroup{$rel->{'service'}}}, $rel->{'host'};

	# Association du service concerne au hostgroup fraichement cree
	$servicesByHostgroup{$rel->{'service'}} = [$rel->{'service'}] unless @servicesByHostgroup{$rel->{'service'}};
}

# Pour chacun des hostgroups
for my $hg_id (keys %hostsByHostgroup) {
	my $hosts = $hostsByHostgroup{$hg_id};

	# Pour chacun des hostgroups
	for my $hg_id_cmp (keys %hostsByHostgroup) {
		my $hosts_cmp = $hostsByHostgroup{$hg_id_cmp};

		# Si le hostgroup courant est dependant des memes services que le second hostgroup courant,
		# alors un troisieme hostgroup. Il aura pour nom la concatenation des deux premiers, contiendra
		# l'ensemble des hosts reunis et sera donc dependant des deux services
		if($hg_id != $hg_id_cmp && join('', sort @$hosts) eq join('', sort @$hosts_cmp)) {

			# Pour chacun des service du second, on l'ajoute au premier, qui sera transforme
			for(my $i = 0; $i < @{$servicesByHostgroup{$hg_id_cmp}}; $i++) {
				push @{$servicesByHostgroup{$hg_id}}, $servicesByHostgroup{$hg_id_cmp}[$i];
				delete $servicesByHostgroup{$hg_id_cmp}[$i];
			}
		}
	}
}

# Pour chaque hostgroup
for my $hg_id (keys %hostsByHostgroup) {

	# Un hostgroup fusionne a entraine un hosgroup vide
	next if @{$servicesByHostgroup{$hg_id}} == 0;

	$mysql->do("LOCK TABLES
		service READ,
		hostgroup WRITE,
		hostgroup_relation WRITE,
		host_service_relation WRITE");

	$mysql->do("SET AUTOCOMMIT = 0");
	my $hg_name = 'AUTO';

	# Pour chacun des hostgroups, on retrouve le nom des services associes et on
	# cree son nom a partir de la concatenation des leurs
	for (@{$servicesByHostgroup{$hg_id}}) {
		my $sql = $mysql->prepare("SELECT service_description name
			FROM service
			WHERE service_id=$_");
		$sql->execute;

		my $service_name = $sql->fetchrow_hashref->{'name'};
		$service_name =~ s/(-service|use-)//g;
		$hg_name .= "___$service_name";
	}

	# Creation du hostgroup
	$mysql->do("INSERT INTO hostgroup (
		hg_name,
		hg_alias,
		hg_activate
	) VALUES (
		'$hg_name',
		'AUTO',
		'1'
	)");

	# Recuperation de l'ID genere pour ce nouveau groupe, afin de pouvoir cree
	# ses relations
	my $sql = $mysql->prepare("SELECT LAST_INSERT_ID() id");
	$sql->execute;
	my $real_hg_id = $sql->fetchrow_hashref->{'id'};

	# Creation des liaisons entre les hosts et le hostgroup
	for (@{$hostsByHostgroup{$hg_id}}) {
		$mysql->do("INSERT INTO hostgroup_relation (
			hostgroup_hg_id,
			host_host_id
		) VALUES (
			$real_hg_id,
			$_
		)");
	}

	# Creation des liaisons entre les services et le hostgroup
	for (@{$servicesByHostgroup{$hg_id}}) {
		$mysql->do("INSERT INTO host_service_relation (
			hostgroup_hg_id,
			service_service_id
		) VALUES (
			$real_hg_id,
			$_
		)");
	}

	# Suppression des relations entre les hosts eux-memes et ces services
	# dont ils vont maintenant heriter depuis leur hostgroup
	for my $h (@{$hostsByHostgroup{$hg_id}}) {
		for my $s (@{$servicesByHostgroup{$hg_id}}) {
			$mysql->do("DELETE FROM host_service_relation
				WHERE host_host_id=$h
				AND service_service_id=$s");
		}
	}

	$mysql->do("COMMIT");
	$mysql->do("SET AUTOCOMMIT = 1");
	$mysql->do("UNLOCK TABLES");
}

$sql->finish;
$mysql->disconnect;
